// Copyright (c) 2016, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

import 'package:quiver/strings.dart' show isBlank;

import '../../utils/angular/properties/properties.dart' show getInt;

/// Provides computation of tabindex for components which actively maintain
/// a tab index.
///
/// Use this if your component needs to suppress its tab index when disabled but
/// you want to support a custom tab index via host element.
abstract class HasTabIndex {
  String get hostTabIndex;
  bool get disabled;

  String _tabIndex = null;

  /// Use this method if you want to manually compute and cache the tab index.
  void updateTabIndex() {
    _tabIndex = _computeTabIndex();
  }

  String get tabIndex {
    return _tabIndex != null ? _tabIndex : _computeTabIndex();
  }

  String _computeTabIndex() {
    if (disabled) {
      return '-1';
    } else if (!isBlank(hostTabIndex)) {
      assert(getInt(hostTabIndex) != null);
      return hostTabIndex;
    } else {
      return '0';
    }
  }
}
